﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace P3_7
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class spriteComp : Microsoft.Xna.Framework.DrawableGameComponent
    {
        protected Texture2D sprTexture;
        protected Rectangle sprRectangle;
        protected Vector2 sprPosition;
        protected Rectangle scrBounds;
        //Для генерирования случайных чисел
        protected Random randNum;
        //Количество шагов, которое пройдет спрайт
        protected int stepNumber;
        //Координата, к которой он будет двигаться
        protected Vector2 newPosition;
        //Цвет спрайта
        protected Color sprColor;
        //Скорость перемещения
        protected Vector2 speed;

        public spriteComp(Game game, ref Texture2D newTexture,
            Rectangle newRectangle, int Seed)
            : base(game)
        {
            sprTexture = newTexture;
            sprRectangle = newRectangle;
            //Инициализируем счетчик
            randNum = new Random(Seed);

            scrBounds = new Rectangle(0, 0,
                game.Window.ClientBounds.Width,
                game.Window.ClientBounds.Height);
            //Устанавливаем стартовую позицию спрайта
            sprPosition.X = (float)randNum.NextDouble() * (scrBounds.Width - sprRectangle.Width);
            sprPosition.Y = (float)randNum.NextDouble() * (scrBounds.Height  - sprRectangle.Height);
            //Колишество шагов равно нулю
            stepNumber = 0;
            //Зададим случайный цвет для придания изображению
            //соответствующего оттенка
            sprColor = new Color((byte)randNum.Next(0, 255), (byte)randNum.Next(0, 255), (byte)randNum.Next(0, 255));
            //Переменная для хранения скорости пока пуста
            speed = new Vector2 ();
            // TODO: Construct any child components here
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }
        //Перемещение спрайта
        public virtual void Move()
        {
            //Если выполнены не все шаги
            if (stepNumber > 0)
            {
                //уменьшим переменную, хранящую число шагов
                stepNumber--;
                //Модифицируем координаты в соответствии с положением
                //объекта относительно желаемой координаты
                //и в соответствии со скоростью
                if (sprPosition.X < newPosition.X) sprPosition.X += speed.X;
                if (sprPosition.X > newPosition.X) sprPosition.X -= speed.X;
                if (sprPosition.Y < newPosition.Y) sprPosition.Y += speed.Y;
                if (sprPosition.Y > newPosition.Y) sprPosition.Y -= speed.Y;
            }
            //Если предыдущий шаг завершен, переменная хранит 0
            if (stepNumber ==0)
            {
                //Установим случайное количество шагов
                stepNumber = randNum.Next(50, 200);
                //Сгененируем случайную целевую позицию
                newPosition.X = (float)randNum.NextDouble() * (scrBounds.Width - sprRectangle.Width);
                newPosition.Y = (float)randNum.NextDouble() * (scrBounds.Height - sprRectangle.Height);
                //Установим новую скорость
                speed.X = randNum.Next(1, 5);
                speed.Y = randNum.Next(1, 5);
                //Зададим новый цвет спрайта
                sprColor = new Color((byte)randNum.Next(0, 255), (byte)randNum.Next(0, 255), (byte)randNum.Next(0, 255));
                
            }
            //Вызов проверки на допустимость перемещения
            Check();
        }
        //Проверка допустимости перемещения
        void Check()
        {
            if (sprPosition.X < scrBounds.Left)
            {
                sprPosition.X = scrBounds.Left;
            }
            if (sprPosition.X > scrBounds.Width - sprRectangle.Width)
            {
                sprPosition.X = scrBounds.Width - sprRectangle.Width;
            }
            if (sprPosition.Y < scrBounds.Top)
            {
                sprPosition.Y = scrBounds.Top;
            }
            if (sprPosition.Y > scrBounds.Height - sprRectangle.Height)
            {
                sprPosition.Y = scrBounds.Height - sprRectangle.Height;
            }
        }
        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            //Вызов метода для перемещения спрайта
            Move();
            base.Update(gameTime);
        }
        public override void Draw(GameTime gameTime)
        {
            SpriteBatch sprBatch =
                (SpriteBatch)Game.Services.GetService(typeof(SpriteBatch));
            sprBatch.Draw(sprTexture, sprPosition, sprRectangle, sprColor);
            base.Draw(gameTime);
        }
    }
}